@@ -1,9 +1,12 @@
 """For deserializing the json responses"""
 import abc
 import math
+import random
 from collections import OrderedDict
 import re
 
+from dateutil.parser import parse
+
 from blaseball_mike import database, tables
 
 
@@ -36,6 +39,49 @@ class GlobalEvent(Base):
         return [cls(event) for event in events]
 
 
+class SimulationData(Base):
+
+    @classmethod
+    def load(cls):
+        return cls(database.get_simulation_data())
+
+    @property
+    def league(self):
+        if self._league:
+            return self._league
+        self._league = League.load_by_id(self._league_id)
+        return self._league
+
+    @league.setter
+    def league(self, value):
+        self._league = None
+        self._league_id = value
+
+    @property
+    def next_election_end(self):
+        return self._next_election_end
+
+    @next_election_end.setter
+    def next_election_end(self, value):
+        self._next_election_end = parse(value)
+
+    @property
+    def next_phase_time(self):
+        return self._next_phase_time
+
+    @next_phase_time.setter
+    def next_phase_time(self, value):
+        self._next_phase_time = parse(value)
+
+    @property
+    def next_season_start(self):
+        return self._next_season_start
+
+    @next_season_start.setter
+    def next_season_start(self, value):
+        self._next_season_start = parse(value)
+
+
 class Player(Base):
 
     @classmethod
@@ -104,6 +150,22 @@ class Player(Base):
                       math.sin(math.pi * (2 / (6 + round(10 * self.buoyancy)) * (day - 1) + 0.5)) -
                       self.pressurization + self.cinnamon)
 
+    @property
+    def soulscream(self):
+        letters = ["A", "E", "I", "O", "U", "X", "H", "A", "E", "I"]
+        stats = [self.pressurization, self.divinity, self.tragicness, self.shakespearianism, self.ruthlessness]
+
+        scream = []
+        for r in range(self.soul):
+            sub_scream = []
+            i = 10 ** -r
+            for s in stats:
+                c = math.floor((s % i) / i * 10)
+                sub_scream.append(letters[c])
+            scream.extend(sub_scream + sub_scream + [sub_scream[0]])
+
+        return ''.join(scream)
+
     @property
     def blood(self):
         return tables.Blood(self._blood)
@@ -168,12 +230,13 @@ class Player(Base):
     def game_attr(self, value):
         self._game_attr = value
 
-    def simulated_copy(self, overrides=None, multipliers=None, buffs=None):
+    def simulated_copy(self, overrides=None, multipliers=None, buffs=None, reroll=None):
         """
         Return a copy of this player with adjusted stats (ie to simulate blessings)
         `overrides` is a dict where the key specifies an attribute to completely overwrite with new value.
         `multipliers` is a dict where key specifies attr to multiply by value
         `buffs` is a dict where key specifies attr to add value
+        `reroll` is a dict where the key specifies attr to reroll (value is unused)
 
         `batting_rating`, `pitching_rating`, `baserunning_rating`, `defense_rating`, and `overall_rating`
         can additionally be passed to `multipliers` and `buffs` to automatically multiply the appropriate
@@ -182,6 +245,7 @@ class Player(Base):
         overrides = overrides or {}
         multipliers = multipliers or {}
         buffs = buffs or {}
+        reroll = reroll or {}
         original_json = self.json()
 
         for override_key, override_value in overrides.items():
@@ -251,6 +315,38 @@ class Player(Base):
             elif b_key in original_json:
                 original_json[b_key] = max(0.01, original_json[b_key] + b_val)
 
+        for r_key, _ in reroll.items():
+            if r_key in ('batting_rating', 'overall_rating'):
+                original_json['tragicness'] = random.uniform(0.01, 0.99)
+                original_json['patheticism'] = random.uniform(0.01, 0.99)
+                original_json['thwackability'] = random.uniform(0.01, 0.99)
+                original_json['divinity'] = random.uniform(0.01, 0.99)
+                original_json['moxie'] = random.uniform(0.01, 0.99)
+                original_json['musclitude'] = random.uniform(0.01, 0.99)
+                original_json['martyrdom'] = random.uniform(0.01, 0.99)
+            if r_key in ('pitching_rating', 'overall_rating'):
+                original_json['unthwackability'] = random.uniform(0.01, 0.99)
+                original_json['ruthlessness'] = random.uniform(0.01, 0.99)
+                original_json['overpowerment'] = random.uniform(0.01, 0.99)
+                original_json['shakespearianism'] = random.uniform(0.01, 0.99)
+                original_json['coldness'] = random.uniform(0.01, 0.99)
+            if r_key in ('baserunning_rating', 'overall_rating'):
+                original_json['laserlikeness'] = random.uniform(0.01, 0.99)
+                original_json['continuation'] = random.uniform(0.01, 0.99)
+                original_json['baseThirst'] = random.uniform(0.01, 0.99)
+                original_json['indulgence'] = random.uniform(0.01, 0.99)
+                original_json['groundFriction'] = random.uniform(0.01, 0.99)
+            if r_key in ('defense_rating', 'overall_rating'):
+                original_json['omniscience'] = random.uniform(0.01, 0.99)
+                original_json['tenaciousness'] = random.uniform(0.01, 0.99)
+                original_json['watchfulness'] = random.uniform(0.01, 0.99)
+                original_json['anticapitalism'] = random.uniform(0.01, 0.99)
+                original_json['chasiness'] = random.uniform(0.01, 0.99)
+            if r_key in ('tragicness', 'patheticism'):
+                original_json[r_key] = random.uniform(0.01, 0.99)
+            elif r_key in original_json:
+                original_json[r_key] = random.uniform(0.01, 0.99)
+
         return Player(original_json)
 
 
@@ -473,6 +569,18 @@ class League(Base):
             self._teams.update(subleague.teams)
         return self._teams
 
+    @property
+    def tiebreakers(self):
+        if self._tiebreakers:
+            return self._tiebreakers
+        self._tiebreakers = Tiebreaker.load(self._tiebreakers_id)
+        return self._tiebreakers
+
+    @tiebreakers.setter
+    def tiebreakers(self, value):
+        self._tiebreakers = None
+        self._tiebreakers_id = value
+
 
 class Game(Base):
 
@@ -713,6 +821,18 @@ class Game(Base):
     def inning(self, value):
         self._inning = value + 1
 
+    @property
+    def statsheet(self):
+        if self._statsheet:
+            return self._statsheet
+        self._statsheet = GameStatsheet.load(self._statsheet_id)[self._statsheet_id]
+        return self._statsheet
+
+    @statsheet.setter
+    def statsheet(self, value):
+        self._statsheet = None
+        self._statsheet_id = value
+
 
 class DecreeResult(Base):
 
@@ -1027,6 +1147,42 @@ class Season(Base):
         self._standings = None
         self._standings_id = value
 
+    @property
+    def stats(self):
+        if self._stats:
+            return self._stats
+        self._stats = SeasonStatsheet.load(self._stats_id)[self._stats_id]
+        return self._stats
+
+    @stats.setter
+    def stats(self, value):
+        self._stats_id = value
+        self._stats = None
+
+
+class Tiebreaker(Base):
+
+    @classmethod
+    def load(cls, id):
+        tiebreakers = database.get_tiebreakers(id)
+        return {
+            id_: cls(tiebreaker) for (id_, tiebreaker) in tiebreakers.items()
+        }
+
+    @property
+    def order(self):
+        if self._order:
+            return self._order
+        self._order = OrderedDict()
+        for id_ in self._order_ids:
+            self._order[id_] = Team.load(id_)
+        return self._order
+
+    @order.setter
+    def order(self, value):
+        self._order = None
+        self._order_ids = value
+
 
 class Idol(Base):
 
@@ -1035,12 +1191,117 @@ class Idol(Base):
         idols = database.get_idols()
         idols_dict = OrderedDict()
         for idol in idols:
-            idols_dict[idol['id']] = cls(idol)
+            idols_dict[idol['playerId']] = cls(idol)
         return idols_dict
 
     @property
     def player(self):
-        if self._player:
+        if getattr(self, '_player', None):
             return self._player
         self._player = Player.load_one(self.player_id)
         return self._player
+
+
+class PlayerStatsheet(Base):
+
+    @classmethod
+    def load(cls, ids):
+        stats = database.get_player_statsheets(ids)
+        stats_dict = OrderedDict()
+        for k, v in stats.items():
+            stats_dict[k] = cls(v)
+        return stats_dict
+
+
+class TeamStatsheet(Base):
+
+    @classmethod
+    def load(cls, ids):
+        stats = database.get_team_statsheets(ids)
+        stats_dict = OrderedDict()
+        for k, v in stats.items():
+            stats_dict[k] = cls(v)
+        return stats_dict
+
+    @property
+    def player_stats(self):
+        if self._player_stats:
+            return self._player_stats
+        self._player_stats = list(PlayerStatsheet.load(self._player_stat_ids).values())
+        return self._player_stats
+
+    @player_stats.setter
+    def player_stats(self, ids):
+        self._player_stat_ids = ids
+        self._player_stats = None
+
+
+class SeasonStatsheet(Base):
+
+    @classmethod
+    def load(cls, ids):
+        stats = database.get_season_statsheets(ids)
+        stats_dict = OrderedDict()
+        for k, v in stats.items():
+            stats_dict[k] = cls(v)
+        return stats_dict
+
+    def load_by_season(cls, season):
+        """Season is 1 indexed."""
+        season = Season.load(season)
+        return season.stats
+
+    @property
+    def team_stats(self):
+        if self._team_stats:
+            return self._team_stats
+        self._team_stats = list(TeamStatsheet.load(self._team_stat_ids).values())
+        return self._team_stats
+
+    @team_stats.setter
+    def team_stats(self, value):
+        self._team_stats = None
+        self._team_stat_ids = value
+
+
+class GameStatsheet(Base):
+
+    @classmethod
+    def load(cls, ids):
+        stats = database.get_game_statsheets(ids)
+        stats_dict = OrderedDict()
+        for k, v in stats.items():
+            stats_dict[k] = cls(v)
+        return stats_dict
+
+    @classmethod
+    def load_by_day(cls, season, day):
+        games = Game.load_by_day(season, day)
+        return {k: g.statsheet for k, g in games.items()}
+
+    def team_stats(self):
+        if getattr(self, '_team_stats', None):
+            return self._team_stats
+        self._team_stats = TeamStatsheet.load([
+            self._home_team_stats_id,
+            self._away_team_stats_id,
+        ])
+        return self._team_stats
+
+    @property
+    def away_team_stats(self):
+        return self.team_stats()[self._away_team_stats_id]
+
+    @away_team_stats.setter
+    def away_team_stats(self, value):
+        self._away_team_stats_id = value
+        self._team_stats = None
+
+    @property
+    def home_team_stats(self):
+        return self.team_stats()[self._home_team_stats_id]
+
+    @home_team_stats.setter
+    def home_team_stats(self, value):
+        self._home_team_stats_id = value
+        self._team_stats = None
